@@ -25,6 +25,7 @@ from typing import (
 )
 
 import pytest
+from pytest import Function, Item, Session
 
 if sys.version_info >= (3, 8):
     from typing import Literal
@@ -55,22 +56,6 @@ SubRequest = Any  # pytest < 7.0
 class Mode(str, enum.Enum):
     AUTO = "auto"
     STRICT = "strict"
-    LEGACY = "legacy"
-
-
-LEGACY_MODE = DeprecationWarning(
-    "The 'asyncio_mode' default value will change to 'strict' in future, "
-    "please explicitly use 'asyncio_mode=strict' or 'asyncio_mode=auto' "
-    "in pytest configuration file."
-)
-
-LEGACY_ASYNCIO_FIXTURE = (
-    "'@pytest.fixture' is applied to {name} "
-    "in 'legacy' mode, "
-    "please replace it with '@pytest_asyncio.fixture' as a preparation "
-    "for switching to 'strict' mode (or use 'auto' mode to seamlessly handle "
-    "all these fixtures as asyncio-driven)."
-)
 
 
 ASYNCIO_MODE_HELP = """\
@@ -78,8 +63,6 @@ ASYNCIO_MODE_HELP = """\
 'strict' - for autoprocessing disabling (useful if different async frameworks \
 should be tested together, e.g. \
 both pytest-asyncio and pytest-trio are used in the same project)
-'legacy' - for keeping compatibility with pytest-asyncio<0.17: \
-auto-handling is disabled but pytest_asyncio.fixture usage is not enforced
 """
 
 
@@ -95,7 +78,7 @@ def pytest_addoption(parser: Parser, pluginmanager: PytestPluginManager) -> None
     parser.addini(
         "asyncio_mode",
         help="default value for --asyncio-mode",
-        default="legacy",
+        default="strict",
     )
 
 
@@ -106,11 +89,10 @@ def fixture(
     scope: "Union[_ScopeName, Callable[[str, Config], _ScopeName]]" = ...,
     params: Optional[Iterable[object]] = ...,
     autouse: bool = ...,
-    ids: Optional[
-        Union[
-            Iterable[Union[None, str, float, int, bool]],
-            Callable[[Any], Optional[object]],
-        ]
+    ids: Union[
+        Iterable[Union[str, float, int, bool, None]],
+        Callable[[Any], Optional[object]],
+        None,
     ] = ...,
     name: Optional[str] = ...,
 ) -> FixtureFunction:
@@ -124,11 +106,10 @@ def fixture(
     scope: "Union[_ScopeName, Callable[[str, Config], _ScopeName]]" = ...,
     params: Optional[Iterable[object]] = ...,
     autouse: bool = ...,
-    ids: Optional[
-        Union[
-            Iterable[Union[None, str, float, int, bool]],
-            Callable[[Any], Optional[object]],
-        ]
+    ids: Union[
+        Iterable[Union[str, float, int, bool, None]],
+        Callable[[Any], Optional[object]],
+        None,
     ] = ...,
     name: Optional[str] = None,
 ) -> FixtureFunctionMarker:
@@ -139,7 +120,7 @@ def fixture(
     fixture_function: Optional[FixtureFunction] = None, **kwargs: Any
 ) -> Union[FixtureFunction, FixtureFunctionMarker]:
     if fixture_function is not None:
-        _set_explicit_asyncio_mark(fixture_function)
+        _make_asyncio_fixture_function(fixture_function)
         return pytest.fixture(fixture_function, **kwargs)
 
     else:
@@ -151,12 +132,12 @@ def fixture(
         return inner
 
 
-def _has_explicit_asyncio_mark(obj: Any) -> bool:
+def _is_asyncio_fixture_function(obj: Any) -> bool:
     obj = getattr(obj, "__func__", obj)  # instance method maybe?
     return getattr(obj, "_force_asyncio_fixture", False)
 
 
-def _set_explicit_asyncio_mark(obj: Any) -> None:
+def _make_asyncio_fixture_function(obj: Any) -> None:
     if hasattr(obj, "__func__"):
         # instance method, check the function object
         obj = obj.__func__
@@ -187,62 +168,67 @@ def pytest_configure(config: Config) -> None:
         "mark the test as a coroutine, it will be "
         "run using an asyncio event loop",
     )
-    if _get_asyncio_mode(config) == Mode.LEGACY:
-        config.issue_config_time_warning(LEGACY_MODE, stacklevel=2)
+
+    if getattr(pytest, "version_tuple", (0, 0, 0)) < (7,):
+        warnings.warn(
+            "You're using an outdated version of pytest. Newer releases of "
+            "pytest-asyncio will not be compatible with this pytest version. "
+            "Please update pytest to version 7 or later.",
+            DeprecationWarning,
+        )
 
 
-@pytest.mark.tryfirst
+@pytest.hookimpl(tryfirst=True)
 def pytest_report_header(config: Config) -> List[str]:
     """Add asyncio config to pytest header."""
     mode = _get_asyncio_mode(config)
     return [f"asyncio: mode={mode}"]
 
 
-def _preprocess_async_fixtures(config: Config, holder: Set[FixtureDef]) -> None:
+def _preprocess_async_fixtures(
+    config: Config,
+    processed_fixturedefs: Set[FixtureDef],
+) -> None:
     asyncio_mode = _get_asyncio_mode(config)
     fixturemanager = config.pluginmanager.get_plugin("funcmanage")
     for fixtures in fixturemanager._arg2fixturedefs.values():
         for fixturedef in fixtures:
-            if fixturedef is holder:
-                continue
             func = fixturedef.func
-            if not _is_coroutine_or_asyncgen(func):
-                # Nothing to do with a regular fixture function
+            if fixturedef in processed_fixturedefs or not _is_coroutine_or_asyncgen(
+                func
+            ):
                 continue
-            if not _has_explicit_asyncio_mark(func):
-                if asyncio_mode == Mode.AUTO:
-                    # Enforce asyncio mode if 'auto'
-                    _set_explicit_asyncio_mark(func)
-                elif asyncio_mode == Mode.LEGACY:
-                    _set_explicit_asyncio_mark(func)
-                    try:
-                        code = func.__code__
-                    except AttributeError:
-                        code = func.__func__.__code__
-                    name = (
-                        f"<fixture {func.__qualname__}, file={code.co_filename}, "
-                        f"line={code.co_firstlineno}>"
-                    )
-                    warnings.warn(
-                        LEGACY_ASYNCIO_FIXTURE.format(name=name),
-                        DeprecationWarning,
-                    )
+            if not _is_asyncio_fixture_function(func) and asyncio_mode == Mode.STRICT:
+                # Ignore async fixtures without explicit asyncio mark in strict mode
+                # This applies to pytest_trio fixtures, for example
+                continue
+            _make_asyncio_fixture_function(func)
+            _inject_fixture_argnames(fixturedef)
+            _synchronize_async_fixture(fixturedef)
+            assert _is_asyncio_fixture_function(fixturedef.func)
+            processed_fixturedefs.add(fixturedef)
 
-            to_add = []
-            for name in ("request", "event_loop"):
-                if name not in fixturedef.argnames:
-                    to_add.append(name)
 
-            if to_add:
-                fixturedef.argnames += tuple(to_add)
+def _inject_fixture_argnames(fixturedef: FixtureDef) -> None:
+    """
+    Ensures that `request` and `event_loop` are arguments of the specified fixture.
+    """
+    to_add = []
+    for name in ("request", "event_loop"):
+        if name not in fixturedef.argnames:
+            to_add.append(name)
+    if to_add:
+        fixturedef.argnames += tuple(to_add)
 
-            if inspect.isasyncgenfunction(func):
-                fixturedef.func = _wrap_asyncgen(func)
-            elif inspect.iscoroutinefunction(func):
-                fixturedef.func = _wrap_async(func)
 
-            assert _has_explicit_asyncio_mark(fixturedef.func)
-            holder.add(fixturedef)
+def _synchronize_async_fixture(fixturedef: FixtureDef) -> None:
+    """
+    Wraps the fixture function of an async fixture in a synchronous function.
+    """
+    if inspect.isasyncgenfunction(fixturedef.func):
+        _wrap_asyncgen_fixture(fixturedef)
+    elif inspect.iscoroutinefunction(fixturedef.func):
+        _wrap_async_fixture(fixturedef)
 
 
 def _add_kwargs(
@@ -260,14 +246,38 @@ def _add_kwargs(
     return ret
 
 
-def _wrap_asyncgen(func: Callable[..., AsyncIterator[_R]]) -> Callable[..., _R]:
-    @functools.wraps(func)
+def _perhaps_rebind_fixture_func(
+    func: _T, instance: Optional[Any], unittest: bool
+) -> _T:
+    if instance is not None:
+        # The fixture needs to be bound to the actual request.instance
+        # so it is bound to the same object as the test method.
+        unbound, cls = func, None
+        try:
+            unbound, cls = func.__func__, type(func.__self__)  # type: ignore
+        except AttributeError:
+            pass
+        # If unittest is true, the fixture is bound unconditionally.
+        # otherwise, only if the fixture was bound before to an instance of
+        # the same type.
+        if unittest or (cls is not None and isinstance(instance, cls)):
+            func = unbound.__get__(instance)  # type: ignore
+    return func
+
+
+def _wrap_asyncgen_fixture(fixturedef: FixtureDef) -> None:
+    fixture = fixturedef.func
+
+    @functools.wraps(fixture)
     def _asyncgen_fixture_wrapper(
         event_loop: asyncio.AbstractEventLoop, request: SubRequest, **kwargs: Any
-    ) -> _R:
+    ):
+        func = _perhaps_rebind_fixture_func(
+            fixture, request.instance, fixturedef.unittest
+        )
         gen_obj = func(**_add_kwargs(func, kwargs, event_loop, request))
 
-        async def setup() -> _R:
+        async def setup():
             res = await gen_obj.__anext__()
             return res
 
@@ -290,55 +300,74 @@ def _wrap_asyncgen(func: Callable[..., AsyncIterator[_R]]) -> Callable[..., _R]:
         request.addfinalizer(finalizer)
         return result
 
-    return _asyncgen_fixture_wrapper
+    fixturedef.func = _asyncgen_fixture_wrapper
 
 
-def _wrap_async(func: Callable[..., Awaitable[_R]]) -> Callable[..., _R]:
-    @functools.wraps(func)
+def _wrap_async_fixture(fixturedef: FixtureDef) -> None:
+    fixture = fixturedef.func
+
+    @functools.wraps(fixture)
     def _async_fixture_wrapper(
         event_loop: asyncio.AbstractEventLoop, request: SubRequest, **kwargs: Any
-    ) -> _R:
-        async def setup() -> _R:
+    ):
+        func = _perhaps_rebind_fixture_func(
+            fixture, request.instance, fixturedef.unittest
+        )
+
+        async def setup():
             res = await func(**_add_kwargs(func, kwargs, event_loop, request))
             return res
 
         return event_loop.run_until_complete(setup())
 
-    return _async_fixture_wrapper
+    fixturedef.func = _async_fixture_wrapper
 
 
 _HOLDER: Set[FixtureDef] = set()
 
 
-@pytest.mark.tryfirst
+@pytest.hookimpl(tryfirst=True)
 def pytest_pycollect_makeitem(
     collector: Union[pytest.Module, pytest.Class], name: str, obj: object
 ) -> Union[
-    None, pytest.Item, pytest.Collector, List[Union[pytest.Item, pytest.Collector]]
+    pytest.Item, pytest.Collector, List[Union[pytest.Item, pytest.Collector]], None
 ]:
     """A pytest hook to collect asyncio coroutines."""
     if not collector.funcnamefilter(name):
         return None
     _preprocess_async_fixtures(collector.config, _HOLDER)
-    if (
-        _is_coroutine(obj)
-        or _is_hypothesis_test(obj)
-        and _hypothesis_test_wraps_coroutine(obj)
-    ):
-        item = pytest.Function.from_parent(collector, name=name)
-        marker = item.get_closest_marker("asyncio")
-        if marker is not None:
-            return list(collector._genfunctions(name, obj))
-        else:
-            if _get_asyncio_mode(item.config) == Mode.AUTO:
-                # implicitly add asyncio marker if asyncio mode is on
-                ret = list(collector._genfunctions(name, obj))
-                for elem in ret:
-                    elem.add_marker("asyncio")
-                return ret  # type: ignore[return-value]
     return None
 
 
+def pytest_collection_modifyitems(
+    session: Session, config: Config, items: List[Item]
+) -> None:
+    """
+    Marks collected async test items as `asyncio` tests.
+
+    The mark is only applied in `AUTO` mode. It is applied to:
+
+      - coroutines
+      - staticmethods wrapping coroutines
+      - Hypothesis tests wrapping coroutines
+
+    """
+    if _get_asyncio_mode(config) != Mode.AUTO:
+        return
+    function_items = (item for item in items if isinstance(item, Function))
+    for function_item in function_items:
+        function = function_item.obj
+        if isinstance(function, staticmethod):
+            # staticmethods need to be unwrapped.
+            function = function.__func__
+        if (
+            _is_coroutine(function)
+            or _is_hypothesis_test(function)
+            and _hypothesis_test_wraps_coroutine(function)
+        ):
+            function_item.add_marker("asyncio")
+
+
 def _hypothesis_test_wraps_coroutine(function: Any) -> bool:
     return _is_coroutine(function.hypothesis.inner_test)
 
@@ -370,7 +399,9 @@ def pytest_fixture_setup(
         loop = outcome.get_result()
         policy = asyncio.get_event_loop_policy()
         try:
-            old_loop = policy.get_event_loop()
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore", DeprecationWarning)
+                old_loop = policy.get_event_loop()
             if old_loop is not loop:
                 old_loop.close()
         except RuntimeError:
@@ -429,8 +460,8 @@ def wrap_in_sync(
         func = raw_func
 
     @functools.wraps(func)
-    def inner(**kwargs):
-        coro = func(**kwargs)
+    def inner(*args, **kwargs):
+        coro = func(*args, **kwargs)
         if not inspect.isawaitable(coro):
             pyfuncitem.warn(
                 pytest.PytestWarning(
